import 'package:flutter/material.dart';
import 'dart:ui';

enum NiceDialogType
{
  NONE,
  INFO,
  WARNING,
  ERROR,
  MULTIPLE_OPTIONS
}

class NiceDialog extends StatelessWidget{

  NiceDialogType type;

  Color typeInfoColor;
  Color typeWarningColor;
  Color typeErrorColor;
  Color typeMultipleOptionsColor;

  Color backgroundColor;
  double elevation;
  ShapeBorder shape;

  Widget header;
  Color headerColor;
  Widget headerIcon;
  IconData headerIconData;
  double headerIconSize;
  Color headerIconColor;
  double headerPadding;
  double borderRadius;

  Widget title;
  String titleText;
  Color titleColor;

  Widget content;
  String message;
  Color messageColor;

  Widget btnOk;
  String btnOkText;
  Color btnOkColor;
  VoidCallback btnOkAction;

  Widget btnCancel;
  String btnCancelText;
  Color btnCancelColor;
  VoidCallback btnCancelAction;

  Widget btnNeutral;
  String btnNeutralText;
  Color btnNeutralColor;
  VoidCallback btnNeutralAction;


  NiceDialog({
    this.type, this.typeInfoColor, this.typeWarningColor,
    this.typeErrorColor, this.typeMultipleOptionsColor, this.backgroundColor,
    this.elevation, this.shape, this.header, this.headerColor,
    this.headerIcon, this.headerIconData, this.headerIconSize,
    this.headerIconColor, this.headerPadding, this.borderRadius, this.title,
    this.titleText, this.titleColor, this.content, this.message,
    this.messageColor, this.btnOk, this.btnOkText, this.btnOkColor,
    this.btnOkAction, this.btnCancel, this.btnCancelText, this.btnCancelColor,
    this.btnCancelAction, this.btnNeutral, this.btnNeutralText,
    this.btnNeutralColor, this.btnNeutralAction,
  }){
    borderRadius ??= 16;
    headerPadding ??= 8;
    headerIconSize ??= 48;

    shape ??= RoundedRectangleBorder(
        borderRadius: BorderRadius.all(Radius.circular(borderRadius))
    );

    typeInfoColor ??= Colors.blue;
    typeWarningColor ??= Colors.orange;
    typeErrorColor ??= Colors.red.shade800;
    typeMultipleOptionsColor ??= Colors.green;
    type ??= NiceDialogType.NONE;


    headerIconData ??= (_getDialogTypeIconData() ?? Icons.info);
    headerIconColor ??= Colors.white;
    headerColor ??= (_getDialogTypeColor() ?? Colors.blue);
    headerIcon ??= Icon(headerIconData, color: headerIconColor, size: headerIconSize,);
  }

  Color _getDialogTypeColor() {
    switch(type){
      case NiceDialogType.INFO: return typeInfoColor;
      case NiceDialogType.WARNING: return typeWarningColor;
      case NiceDialogType.ERROR: return typeErrorColor;
      case NiceDialogType.MULTIPLE_OPTIONS: return typeMultipleOptionsColor;
      case NiceDialogType.NONE:
      default:
        return null;
    }
  }

  IconData _getDialogTypeIconData() {
    switch(type){
      case NiceDialogType.INFO: return Icons.info;
      case NiceDialogType.WARNING: return Icons.warning;
      case NiceDialogType.ERROR: return Icons.error;
      case NiceDialogType.MULTIPLE_OPTIONS: return Icons.list;
      case NiceDialogType.NONE:
      default:
        return null;
    }
  }

  void _checkParams(BuildContext context){

    titleColor ??= Theme.of(context).textTheme.headline6.color;
    if(title == null && titleText != null)
      title = Text(titleText, textAlign: TextAlign.center,
        style: TextStyle(fontSize: 18, color: titleColor, fontWeight: FontWeight.bold),);

    messageColor ??= Theme.of(context).textTheme.subtitle2.color;
    if(content == null && message != null)
      content = Text(message, textAlign: TextAlign.center,
        style: TextStyle(color: messageColor),);

    backgroundColor ??= DialogTheme.of(context).backgroundColor;
    elevation ??= DialogTheme.of(context).elevation;
    Color accentColor = Theme.of(context).accentColor;
    btnOkColor ??= accentColor;
    btnCancelColor ??= accentColor;
    btnNeutralColor ??= accentColor;
  }

  Widget _buildHeader(){
    if(header != null) return header;
    return Container(
      alignment: Alignment.center,
      padding: EdgeInsets.all(headerPadding),
      decoration: ShapeDecoration(
        color: headerColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(borderRadius),
              topRight: Radius.circular(borderRadius))
        )
      ),
      child: headerIcon,
    );
  }

  Widget _buildTitle(){
    return Container(
      margin: EdgeInsets.only(top: 10),
      child: title,
    );
  }

  Widget _buildContent(){
    return Container(
      margin: EdgeInsets.only(top: 10),
      child: content,
    );
  }

  Widget _buildButtons(BuildContext context){
    if(btnNeutral == null && btnNeutralText != null)
      btnNeutral = FlatButton(
        textColor: btnNeutralColor ?? Theme.of(context).accentColor,
        child: Text(btnNeutralText),
        onPressed: btnNeutralAction ?? () => Navigator.of(context).pop(),
      );
    if(btnCancel == null && btnCancelText != null)
      btnCancel = FlatButton(
        textColor: btnCancelColor ?? Theme.of(context).accentColor,
        child: Text(btnCancelText),
        onPressed: btnCancelAction ?? () => Navigator.of(context).pop(),
      );
    if(btnOk == null && btnOkText != null)
      btnOk = FlatButton(
        textColor: btnOkColor ?? Theme.of(context).accentColor,
        child: Text(btnOkText),
        onPressed: btnOkAction ?? () => Navigator.of(context).pop(),
      );
    return Wrap(
      alignment: WrapAlignment.center,
      spacing: 8,
      children: <Widget>[
        btnNeutral != null ? btnNeutral : Container(),
        btnCancel != null ? btnCancel : Container(),
        btnOk != null ? btnOk : Container(),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    _checkParams(context);
    return Dialog(
      backgroundColor: backgroundColor,
      elevation: elevation,
      shape: shape,
      child: Container(
        width: MediaQueryData.fromWindow(window).size.shortestSide * 60 / 100,
        child: Scrollbar(
            child: SingleChildScrollView(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  _buildHeader(),
                  Padding(
                    padding: EdgeInsets.only(left: 16, right: 16),
                    child: _buildTitle(),
                  ),
                  Padding(
                    padding: EdgeInsets.only(left: 16, right: 16),
                    child: _buildContent(),
                  ),
                  Divider(color: Colors.transparent,),
                  _buildButtons(context),
                ],
              ),
            )
        ),
      ),
    );
  }
}