import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';

class SwitchPreference extends StatefulWidget{
  final String title;
  final String summary;
  final String summaryPrefix;
  final String summarySuffix;
  final String localKey;
  final bool defaultValue;
  final Widget leading;
  final bool enabled;
  final bool useValueAsSummary;
  final Function(bool value) onChange;

  SwitchPreference(this.title, this.localKey,
      {
        this.summary,
        this.summaryPrefix,
        this.summarySuffix,
        this.defaultValue = false,
        this.leading,
        this.enabled = true,
        this.useValueAsSummary = false,
        this.onChange,
      });

  _SwitchPreferenceState createState() => _SwitchPreferenceState();
}

class _SwitchPreferenceState extends State<SwitchPreference> {
  String summary;
  bool value;

  @override
  void initState() {
    super.initState();
    value = SharedPreferencesUtils.getBool(widget.localKey, defaultValue: widget.defaultValue);
    summary =
        (widget.summaryPrefix != null ? widget.summaryPrefix : "") +
            (widget.useValueAsSummary && value != null ? value : widget.summary) +
            (widget.summarySuffix != null ? widget.summarySuffix : "");
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: Text(widget.title),
      subtitle: summary != null ? Text(summary) : null,
      leading: widget.leading,
      enabled: widget.enabled,
      onTap: () => onChange(!value),
      trailing: Switch.adaptive(
        value: value,
        onChanged: (value) => onChange(value),
      ),
    );
  }

  void onChange(value){
    SharedPreferencesUtils.putBool(widget.localKey, this.value = value);
    setState(() {});
    if(widget.onChange != null) widget.onChange(value);
  }
}