import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/scheduler.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:adeptutils/adeptutils.dart';
import 'dart:ui' as ui show BoxHeightStyle, BoxWidthStyle;

enum CustomButton{
  NONE,
  ALWAYS,
  ON_TEXT,
}

enum CustomButtonDefaultAction{
  NONE,
  CLEAR,
  PASSWORD_TOGGLE,
}


class AdeptTextField extends StatefulWidget {
  Key key;
  AdeptTextEditingController controller;
  FocusNode focusNode;
  InputDecoration decoration;
  TextInputType keyboardType;
  TextInputAction textInputAction;
  TextCapitalization textCapitalization;
  TextStyle style;
  TextAlign textAlign;
  TextDirection textDirection;
  bool autofocus;
  bool obscureText;
  bool autocorrect;
  int maxLines;
  int maxLength;
  bool maxLengthEnforced;
  ValueChanged<String> onChanged;
  VoidCallback onEditingComplete;
  ValueChanged<String> onSubmitted;
  List<TextInputFormatter> inputFormatters;
  bool enabled;
  double cursorWidth;
  Radius cursorRadius;
  Color cursorColor;
  Brightness keyboardAppearance;
  EdgeInsets scrollPadding;
  bool enableInteractiveSelection;
  GestureTapCallback onTap;
  StrutStyle strutStyle;
  TextAlignVertical textAlignVertical;
  bool readOnly;
  ToolbarOptions toolbarOptions;
  bool showCursor;
  String obscuringCharacter;
  SmartDashesType smartDashesType;
  SmartQuotesType smartQuotesType;
  bool enableSuggestions;
  int minLines;
  bool expands;
  AppPrivateCommandCallback onAppPrivateCommand;
  double cursorHeight;
  ui.BoxHeightStyle selectionHeightStyle;
  ui.BoxWidthStyle selectionWidthStyle;
  DragStartBehavior dragStartBehavior;
  MouseCursor mouseCursor;
  InputCounterWidgetBuilder buildCounter;
  ScrollController scrollController;
  ScrollPhysics scrollPhysics;
  Iterable<String> autofillHints;
  String restorationId;

  CustomButton customButton;
  CustomButtonDefaultAction customButtonDefaultAction;
  VoidCallback customButtonAction;
  Color customButtonColor;
  IconData customButtonIcon;
  double customButtonIconSize;
  double customButtonIconPadding;
  bool clearErrorOnTextChange;

  AdeptTextField({
    this.key,
    this.controller,
    this.focusNode,
    this.decoration = const InputDecoration(),
    this.keyboardType,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.textAlign = TextAlign.start,
    this.textDirection,
    this.autofocus = false,
    this.obscureText = false,
    this.autocorrect = true,
    this.maxLines = 1,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.inputFormatters,
    this.enabled,
    this.cursorWidth = 2.0,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.enableInteractiveSelection = true,
    this.onTap,
    this.strutStyle,
    this.textAlignVertical,
    this.readOnly = false,
    this.toolbarOptions,
    this.showCursor,
    this.obscuringCharacter = '•',
    this.smartDashesType,
    this.smartQuotesType,
    this.enableSuggestions = true,
    this.minLines,
    this.expands = false,
    this.onAppPrivateCommand,
    this.cursorHeight,
    this.selectionHeightStyle = ui.BoxHeightStyle.tight,
    this.selectionWidthStyle = ui.BoxWidthStyle.tight,
    this.dragStartBehavior = DragStartBehavior.start,
    this.mouseCursor,
    this.buildCounter,
    this.scrollController,
    this.scrollPhysics,
    this.autofillHints,
    this.restorationId,

    this.customButton = CustomButton.NONE,
    this.customButtonDefaultAction = CustomButtonDefaultAction.NONE,
    this.customButtonAction,
    this.customButtonColor,
    this.customButtonIcon,
    this.customButtonIconSize,
    this.customButtonIconPadding,
    this.clearErrorOnTextChange = true,
  }) {
    controller ??= AdeptTextEditingController();
    customButtonIconSize ??= 24;
    customButtonIconPadding ??= 8;
  }

  @override
  _AdeptTextFieldState createState() => _AdeptTextFieldState();
}

class _AdeptTextFieldState extends State<AdeptTextField>{

  bool hasText = false;
  bool hasError = false;
  String lastText;

  _AdeptTextFieldState();

  @override
  void initState() {
    super.initState();
    widget.obscureText = widget.customButtonDefaultAction == CustomButtonDefaultAction.PASSWORD_TOGGLE;
    hasText = controllerHasText;
    hasError = widget.controller.hasError;
    lastText = widget.controller.text;
    widget.controller.addListener(onTextChanged);
  }

  void updateDecoration(){
    if(widget.customButton != CustomButton.NONE){
      if(widget.decoration == null) widget.decoration = InputDecoration();
      widget.decoration = widget.decoration.copyWith(
        errorText: widget.controller.error,
        suffixIcon: !hasText ? Container(width: 0, height: 0,) : IconButton(
          icon: getCustomButtonIcon,
          iconSize: widget.customButtonIconSize,
          padding: EdgeInsets.all(widget.customButtonIconPadding),
          onPressed: (){
            if(widget.customButtonAction != null) widget.customButtonAction(); else{
              switch(widget.customButtonDefaultAction){
                case CustomButtonDefaultAction.CLEAR:
                  widget.controller.clear();
                  break;
                case CustomButtonDefaultAction.PASSWORD_TOGGLE:
                  widget.obscureText = !widget.obscureText;
                  setState(() {});
                  break;
                default:
              }
            }
          },
        ),
        suffixText: null,
      );
    }
  }

  bool get controllerHasText {
    bool show = false;
    if(widget.customButton == CustomButton.ALWAYS) show = true;
    if(widget.customButton == CustomButton.ON_TEXT && widget.controller.text != null &&
        widget.controller.text.isNotEmpty) show = true;
    return show;
  }

  void onTextChanged(){
    bool controllerHasText = this.controllerHasText;
    if(controllerHasText != hasText){
      hasText = controllerHasText;
      hasError = widget.controller.hasError;
      setState(() {});
    } else
    if(hasError != widget.controller.hasError){
      hasError = widget.controller.hasError;
      setState(() {});
    }
    if(widget.onChanged != null && lastText != widget.controller.text)
      widget.onChanged(lastText = widget.controller.text);
    lastText = widget.controller.text;
  }

  Icon get getCustomButtonIcon {
    if(widget.customButtonIcon != null) return Icon(widget.customButtonIcon, color: widget.customButtonColor,);
    switch(widget.customButtonDefaultAction){
      case CustomButtonDefaultAction.CLEAR:
        return Icon(Icons.clear, color: widget.customButtonColor,);
      case CustomButtonDefaultAction.PASSWORD_TOGGLE:
        return Icon(widget.obscureText ? Icons.visibility_off : Icons.visibility , color: widget.customButtonColor,);
      default:
    }
    return null;
  }

  @override
  void didUpdateWidget(covariant AdeptTextField oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.controller != widget.controller) {
      oldWidget.controller?.removeListener(onTextChanged);
      widget.controller?.addListener(onTextChanged);
    }
  }

  @override
  Widget build(BuildContext context) {
    updateDecoration();
    return Theme(
      data: Theme.of(context).copyWith(splashColor: Colors.transparent),
      child: TextField(
        key: widget.key,
        controller: widget.controller,
        focusNode: widget.focusNode,
        decoration: widget.decoration,
        keyboardType: widget.keyboardType,
        textInputAction: widget.textInputAction,
        textCapitalization: widget.textCapitalization,
        style: widget.style,
        textAlign: widget.textAlign,
        textDirection: widget.textDirection,
        autofocus : widget.autofocus ,
        obscureText : widget.obscureText ,
        autocorrect : widget.autocorrect ,
        maxLines: widget.maxLines,
        maxLength: widget.maxLength,
        maxLengthEnforced : widget.maxLengthEnforced ,
//        onChanged: widget.onChanged, //onChanged function is managed above in the onTextChanged function
        onEditingComplete: widget.onEditingComplete,
        onSubmitted: widget.onSubmitted,
        inputFormatters: widget.inputFormatters,
        enabled: widget.enabled,
        cursorWidth: widget.cursorWidth,
        cursorRadius: widget.cursorRadius,
        cursorColor: widget.cursorColor,
        keyboardAppearance: widget.keyboardAppearance,
        scrollPadding: widget.scrollPadding,
        enableInteractiveSelection: widget.enableInteractiveSelection,
        onTap: widget.onTap,
        strutStyle: widget.strutStyle,
        textAlignVertical: widget.textAlignVertical,
        readOnly: widget.readOnly,
        toolbarOptions: widget.toolbarOptions,
        showCursor: widget.showCursor,
        obscuringCharacter: widget.obscuringCharacter,
        smartDashesType: widget.smartDashesType,
        smartQuotesType: widget.smartQuotesType,
        enableSuggestions: widget.enableSuggestions,
        minLines: widget.minLines,
        expands: widget.expands,
        onAppPrivateCommand: widget.onAppPrivateCommand,
        cursorHeight: widget.cursorHeight,
        selectionHeightStyle: widget.selectionHeightStyle,
        selectionWidthStyle: widget.selectionWidthStyle,
        dragStartBehavior: widget.dragStartBehavior,
        mouseCursor: widget.mouseCursor,
        buildCounter: widget.buildCounter,
        scrollController: widget.scrollController,
        scrollPhysics: widget.scrollPhysics,
        autofillHints: widget.autofillHints,
        restorationId: widget.restorationId,
      ),
    );
  }

  @override
  void dispose() {
    widget.controller.removeListener(onTextChanged);
    super.dispose();
  }
}